######################################################################################
## Nonparametric Spatial Ordinal Models for clustered periodontal data   	        ##
## R code by Dipankar Bandyopadhyay and Antonio Canale -  Nov/Dec 2013 and May 2014 ##
######################################################################################

#build and load C shared library
system("R CMD SHLIB likelihood.c")
dyn.load("likelihood.so")

#source auxiliary functions
source("auxiliary_function.R")

#load required packages
library(mnormt)
library(tmvtnorm)
library(truncnorm)
library(BDgraph) # note that version 2.7 of BDgraph is needed to sample from the  Gwishart 

# load data
# data y must be a matrix with a line for each subject [nrow(y) = n] and a column for each site location [ncol(y) = m]
# the subject covariates "x" are stored in a matrix with nrow(y) rows and p columns
# the sites covariates "z" are stored in a matrix with ncol(y) rows and p2 columns
# ADJ is an adjacency square matrix of zero and one of dimension m 
y <- read.table("")
x <- read.table("")
z <- read.table("")
ADJ <- read.table("")

#declare MCMC iterations and burn in 
nrep <- 22000
nbi <- 2000

#dimensions
p <- ncol(x)
p2 <- ncol(z)
n <- nrow(y)
m <- ncol(y)	
K <- 5 # number of ordinal categories
a_t <- b_t <- 1/2
D <- diag(apply(ADJ,1,sum))

#prior parameters
betasubj0 <- rep(0, p)
sigma2betasubj <- 20
betasite0 <- rep(0, p2)
sigma2betasite <- 20

#declare all variables
k <- 20 #upper bound for the number of components (for computational reasons)
thetas <-  matrix(0, k, nrep)
sigmas <-  matrix(1, k, nrep)
weights <- array(1/k, dim=c(m,k+1))
xi <- array(1, dim=c(n,m,nrep))
slices <- matrix(1/k, n,m)
alpha <- array(0, dim=c(m,k))
eta <- array(0, dim=c(m,k,n))
tau2 <- rho <- rep(0, nrep)
tau2[1] <- 1
rho[1] <- 0.95
betasubj <- array(0, dim=c(p, nrep))
betasite <- array(0, dim=c(p2, nrep))
a1 <- rep(0, nrep)
gammas <- matrix(1, 3, nrep)
a <- matrix(c(-Inf, -4, 0, 4, 8, Inf), K+1, nrep) #cutoff specification
XX <- t(as.matrix(x)) %*% as.matrix(x)
ZZ <- t(as.matrix(z)) %*% as.matrix(z)
NegSig <- (D - 0.9*ADJ)

# start the mcmc
for(ite in 2:nrep)
{
	#print iteration
	cat(ite," ")

	#new data aumentation for ystar
	ystar <- latentystar(y, xi[,,ite-1], X=x, Z=z, betasubj=betasubj[,ite-1], betasite=betasite[,ite-1], theta=thetas[,ite-1], sigma=sigmas[,ite-1], a=a[,ite-1]) 
	#sample coefficients of the linear regressions explicitly
	#subject specific fixed effect
	ystarj <- ystar - matrix(thetas[,ite-1][xi[,,ite-1]], n, m) - t(matrix(as.double(as.matrix(z) %*% betasite[,ite-1]), m, n))
	Xystarj <- apply(t(as.matrix(x)) %*% ystarj, 1, sum)
	betasubj.post.var <- solve(m * XX + diag(1/sigma2betasubj, dim(XX)))
	betasubj.post.mean <- as.double((Xystarj + betasubj0/sigma2betasubj) %*% betasubj.post.var )
	betasubj[,ite] <- rmnorm(1, mean=betasubj.post.mean, varcov=betasubj.post.var)
	#site specific fixed effect
	ystari <- ystar - matrix(thetas[,ite-1][xi[,,ite-1]], n, m) -  matrix(as.double(as.matrix(x) %*% betasubj[,ite]), n, m)
	Zystari <- apply(t(as.matrix(z)) %*% t(ystari), 1, sum)
	betasite.post.var <- solve(n * ZZ + diag(1/sigma2betasite, dim(ZZ)))
	betasite.post.mean <- as.double((Zystari + betasite0/sigma2betasite) %*% betasite.post.var )
	betasite[,ite] <- rmnorm(1, mean=betasite.post.mean, varcov=betasite.post.var)
	ytilde <- ystar -  matrix(as.double(as.matrix(x) %*% betasubj[,ite-1]), n, m) - t(matrix(as.double(as.matrix(z) %*% betasite[,ite-1]), m, n))

	# nonparametric spatial random effect:
	#update slices
	w <- t(matrix(t(weights)[cbind(c(t(xi[,,ite-1])),rep(1:m, n))],m,n)) #matrice dei pesi n x m
	w[is.na(y)] <- NA
	slice <- matrix(runif(n*m),n,m) * w
	#update cluster indicators
	xi[,,ite] <- labelling3_c(ytilde, slice, weights[,-(k+1)], thetas[,ite-1], sigmas[,ite-1]) 	
	#update atoms
	postmeans <- tapply(c(ytilde), factor(c(xi[,,ite]), levels=1:k), mean)
	postvars <- tapply(c(ytilde), factor(c(xi[,,ite]), levels=1:k), var)
	postclussize <- tapply(c(ytilde), factor(c(xi[,,ite]), levels=1:k), length)
	postclussize[which(is.na(postclussize))] <- 0 
	postmeans[which(is.na(postmeans))] <- 0 
	postvars[which(is.na(postvars))] <- 0 
	sigmas[,ite] <- 1
	thetas[,ite] <- rmnorm(1, mean=c(postclussize*postmeans/(postclussize+1)), varcov=diag((sigmas[,ite]^2)/(postclussize+1)))

	#update weights with probit stick breaking
	out.eta.c <- .C("cycle_eta", as.double(alpha), out=as.double(rep(0,m*k*n)), xi = as.double(t(xi[,,ite])), as.integer(c(n,m,k)))$out # c function
	eta <- array(out.eta.c, dim=c(m, k, n))
	post.var.alpha <- solve(NegSig + diag(n, m))
	post.mean.alpha <- post.var.alpha %*% apply(eta, c(1,2), mean) # m x k matrix
	alpha <- t(rmnorm(k, varcov=post.var.alpha)) + post.mean.alpha
	hmax <- min(k,max(k + 2 - apply(apply(weights,1,cumsum)>(1-min(slice, na.rm=TRUE)),2,sum)))
	#obtain the weights
	weights <- cbind(t(apply(alpha, 1, psb.weights)), rep(0,m))

	#update the covariance matrix
	meanWish <- (D-0.9*ADJ) + (alpha[,1:hmax]%*%t(alpha[,1:hmax]))
	NegSig <- rGWishart(1, m + hmax, meanWish, G = ADJ, start.K = NegSig)[,,1]
}
#
#then calculate your favourite posterior summaries.